@@ -57,6 +57,7 @@
 #include "empathy-group-chat.h"
 //#include "empathy-sound.h"
 #include "empathy-ui-utils.h"
+#include "empathy-about-dialog.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_CHAT_WINDOW, EmpathyChatWindowPriv))
 
@@ -103,6 +104,9 @@ struct _EmpathyChatWindowPriv {
 	GtkWidget             *menu_tabs_left;
 	GtkWidget             *menu_tabs_right;
 	GtkWidget             *menu_tabs_detach;
+	
+	GtkWidget             *menu_help_contents;
+	GtkWidget             *menu_help_about;
 
 	guint                  save_geometry_id;
 };
@@ -168,6 +172,10 @@ static void       chat_window_tabs_right_activate_cb    (GtkWidget             *
 							 EmpathyChatWindow      *window);
 static void       chat_window_detach_activate_cb        (GtkWidget             *menuitem,
 							 EmpathyChatWindow      *window);
+static void       chat_window_help_contents_cb          (GtkWidget             *menuitem,
+							 EmpathyChatWindow      *window);
+static void       chat_window_help_about_cb             (GtkWidget             *menuitem,
+							 EmpathyChatWindow      *window);
 static gboolean   chat_window_delete_event_cb           (GtkWidget             *dialog,
 							 GdkEvent              *event,
 							 EmpathyChatWindow      *window);
@@ -306,6 +314,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_tabs_left", &priv->menu_tabs_left,
 				       "menu_tabs_right", &priv->menu_tabs_right,
 				       "menu_tabs_detach", &priv->menu_tabs_detach,
+				       "menu_help_contents", &priv->menu_help_contents,
+				       "menu_help_about", &priv->menu_help_about,
 				       NULL);
 
 	empathy_glade_connect (glade,
@@ -329,6 +339,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
 			      "menu_tabs_right", "activate", chat_window_tabs_right_activate_cb,
 			      "menu_tabs_detach", "activate", chat_window_detach_activate_cb,
+			      "menu_help_contents", "activate", chat_window_help_contents_cb,
+			      "menu_help_about", "activate", chat_window_help_about_cb,
 			      NULL);
 
 	g_object_unref (glade);
@@ -464,22 +476,12 @@ empathy_chat_window_get_default (void)
 
 	for (l = chat_windows; l; l = l->next) {
 		EmpathyChatWindow *chat_window;
-		GtkWidget        *dialog;
-		GdkWindow        *window;
-		gboolean          visible;
+		GtkWidget         *dialog;
 
 		chat_window = l->data;
 
 		dialog = empathy_chat_window_get_dialog (chat_window);
-		window = dialog->window;
-
-		g_object_get (dialog,
-			      "visible", &visible,
-			      NULL);
-
-		visible = visible && !(gdk_window_get_state (window) & GDK_WINDOW_STATE_ICONIFIED);
-
-		if (visible) {
+		if (empathy_window_get_is_visible (GTK_WINDOW (GTK_WINDOW (dialog)))) {
 			/* Found a visible window on this desktop */
 			return chat_window;
 		}
@@ -1234,6 +1236,20 @@ chat_window_detach_activate_cb (GtkWidget        *menuitem,
 	gtk_widget_show (priv->dialog);
 }
 
+static void
+chat_window_help_contents_cb (GtkWidget         *menuitem,
+			      EmpathyChatWindow *window)
+{
+	//empathy_help_show ();
+}
+
+static void
+chat_window_help_about_cb (GtkWidget         *menuitem,
+			   EmpathyChatWindow *window)
+{
+	empathy_about_dialog_new (GTK_WINDOW (window));
+}
+
 static gboolean
 chat_window_delete_event_cb (GtkWidget        *dialog,
 			     GdkEvent         *event,
@@ -1610,16 +1626,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		chat = empathy_chat_window_find_chat (account, id);
 
 		if (!chat) {
-			MissionControl *mc;
-
-			mc = empathy_mission_control_new ();
-			mission_control_request_channel (mc,
-							 empathy_contact_get_account (contact),
-							 TP_IFACE_CHANNEL_TYPE_TEXT,
-							 empathy_contact_get_handle (contact),
-							 TP_HANDLE_TYPE_CONTACT,
-							 NULL, NULL);
-			g_object_unref (mc);
+			empathy_chat_with_contact_id (account, id);
 			return;
 		}
 
