@@ -2,20 +2,19 @@
 /*
  * Copyright (C) 2007 Collabora Ltd.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
@@ -56,58 +55,58 @@ struct _EmpathyTpChatPriv {
 	DBusGProxy	      *chat_state_iface;
 };
 
-static void             empathy_tp_chat_class_init    (EmpathyTpChatClass        *klass);
-static void             empathy_tp_chat_init          (EmpathyTpChat             *chat);
-static void             tp_chat_finalize              (GObject                   *object);
-static GObject *        tp_chat_constructor           (GType                      type,
-						       guint                      n_props,
-						       GObjectConstructParam     *props);
-static void             tp_chat_get_property          (GObject                   *object,
-						       guint                      param_id,
-						       GValue                    *value,
-						       GParamSpec                *pspec);
-static void             tp_chat_set_property          (GObject                   *object,
-						       guint                      param_id,
-						       const GValue              *value,
-						       GParamSpec                *pspec);
-static void             tp_chat_destroy_cb            (TpChan                    *text_chan,
-						       EmpathyTpChat             *chat);
-static void             tp_chat_closed_cb             (TpChan                    *text_chan,
-						       EmpathyTpChat             *chat);
-static void             tp_chat_received_cb           (DBusGProxy                *text_iface,
-						       guint                      message_id,
-						       guint                      timestamp,
-						       guint                      from_handle,
-						       guint                      message_type,
-						       guint                      message_flags,
-						       gchar                     *message_body,
-						       EmpathyTpChat             *chat);
-static void             tp_chat_sent_cb               (DBusGProxy                *text_iface,
-						       guint                      timestamp,
-						       guint                      message_type,
-						       gchar                     *message_body,
-						       EmpathyTpChat             *chat);
-static void             tp_chat_send_error_cb         (DBusGProxy                *text_iface,
-						       guint                      error_code,
-						       guint                      timestamp,
-						       guint                      message_type,
-						       gchar                     *message_body,
-						       EmpathyTpChat             *chat);
-static void             tp_chat_state_changed_cb      (DBusGProxy                *chat_state_iface,
-						       guint                      handle,
-						       TelepathyChannelChatState  state,
-						       EmpathyTpChat             *chat);
-static EmpathyMessage * tp_chat_build_message         (EmpathyTpChat             *chat,
-						       guint                      type,
-						       guint                      timestamp,
-						       guint                      from_handle,
-						       const gchar               *message_body);
-static void             tp_chat_properties_ready_cb   (TpPropsIface              *props_iface,
-						       EmpathyTpChat             *chat);
-static void             tp_chat_properties_changed_cb (TpPropsIface              *props_iface,
-						       guint                      prop_id,
-						       TpPropsChanged             flag,
-						       EmpathyTpChat             *chat);
+static void             empathy_tp_chat_class_init    (EmpathyTpChatClass    *klass);
+static void             empathy_tp_chat_init          (EmpathyTpChat         *chat);
+static void             tp_chat_finalize              (GObject               *object);
+static GObject *        tp_chat_constructor           (GType                  type,
+						       guint                  n_props,
+						       GObjectConstructParam *props);
+static void             tp_chat_get_property          (GObject               *object,
+						       guint                  param_id,
+						       GValue                *value,
+						       GParamSpec            *pspec);
+static void             tp_chat_set_property          (GObject               *object,
+						       guint                  param_id,
+						       const GValue          *value,
+						       GParamSpec            *pspec);
+static void             tp_chat_destroy_cb            (TpChan                *text_chan,
+						       EmpathyTpChat         *chat);
+static void             tp_chat_closed_cb             (TpChan                *text_chan,
+						       EmpathyTpChat         *chat);
+static void             tp_chat_received_cb           (DBusGProxy            *text_iface,
+						       guint                  message_id,
+						       guint                  timestamp,
+						       guint                  from_handle,
+						       guint                  message_type,
+						       guint                  message_flags,
+						       gchar                 *message_body,
+						       EmpathyTpChat         *chat);
+static void             tp_chat_sent_cb               (DBusGProxy            *text_iface,
+						       guint                  timestamp,
+						       guint                  message_type,
+						       gchar                 *message_body,
+						       EmpathyTpChat         *chat);
+static void             tp_chat_send_error_cb         (DBusGProxy            *text_iface,
+						       guint                  error_code,
+						       guint                  timestamp,
+						       guint                  message_type,
+						       gchar                 *message_body,
+						       EmpathyTpChat         *chat);
+static void             tp_chat_state_changed_cb      (DBusGProxy            *chat_state_iface,
+						       guint                  handle,
+						       TpChannelChatState     state,
+						       EmpathyTpChat         *chat);
+static EmpathyMessage * tp_chat_build_message         (EmpathyTpChat         *chat,
+						       guint                  type,
+						       guint                  timestamp,
+						       guint                  from_handle,
+						       const gchar           *message_body);
+static void             tp_chat_properties_ready_cb   (TpPropsIface          *props_iface,
+						       EmpathyTpChat         *chat);
+static void             tp_chat_properties_changed_cb (TpPropsIface          *props_iface,
+						       guint                  prop_id,
+						       TpPropsChanged         flag,
+						       EmpathyTpChat         *chat);
 enum {
 	PROP_0,
 	PROP_ACCOUNT,
@@ -346,10 +345,31 @@ tp_chat_finalize (GObject *object)
 	chat = EMPATHY_TP_CHAT (object);
 	priv = GET_PRIV (chat);
 
+	if (priv->text_iface) {
+		dbus_g_proxy_disconnect_signal (priv->text_iface, "Received",
+						G_CALLBACK (tp_chat_received_cb),
+						chat);
+		dbus_g_proxy_disconnect_signal (priv->text_iface, "Sent",
+						G_CALLBACK (tp_chat_sent_cb),
+						chat);
+		dbus_g_proxy_disconnect_signal (priv->text_iface, "SendError",
+						G_CALLBACK (tp_chat_send_error_cb),
+						chat);
+	}
+
+	if (priv->chat_state_iface) {
+		dbus_g_proxy_disconnect_signal (priv->chat_state_iface, "ChatStateChanged",
+						G_CALLBACK (tp_chat_state_changed_cb),
+						chat);
+	}
+
 	if (priv->tp_chan) {
 		g_signal_handlers_disconnect_by_func (priv->tp_chan,
 						      tp_chat_destroy_cb,
 						      object);
+		dbus_g_proxy_disconnect_signal (DBUS_G_PROXY (priv->tp_chan), "Closed",
+						G_CALLBACK (tp_chat_closed_cb),
+						chat);
 		if (priv->acknowledge) {
 			empathy_debug (DEBUG_DOMAIN, "Closing channel...");
 			if (!tp_chan_close (DBUS_G_PROXY (priv->tp_chan), &error)) {
@@ -396,11 +416,11 @@ tp_chat_constructor (GType                  type,
 	priv->mc = empathy_mission_control_new ();
 
 	priv->text_iface = tp_chan_get_interface (priv->tp_chan,
-						  TELEPATHY_CHAN_IFACE_TEXT_QUARK);
+						  TP_IFACE_QUARK_CHANNEL_TYPE_TEXT);
 	priv->chat_state_iface = tp_chan_get_interface (priv->tp_chan,
-							TELEPATHY_CHAN_IFACE_CHAT_STATE_QUARK);
+							TP_IFACE_QUARK_CHANNEL_INTERFACE_CHAT_STATE);
 	priv->props_iface = tp_chan_get_interface (priv->tp_chan,
-						   TELEPATHY_PROPS_IFACE_QUARK);
+						   TP_IFACE_QUARK_PROPERTIES_INTERFACE);
 
 	g_signal_connect (priv->tp_chan, "destroy",
 			  G_CALLBACK (tp_chat_destroy_cb),
@@ -558,7 +578,7 @@ empathy_tp_chat_new_with_contact (EmpathyContact *contact)
 	account = empathy_contact_get_account (contact);
 
 	if (mission_control_get_connection_status (mc, account, NULL) != 0) {
-		/* The account is not connected, nothing to do. */
+		/* The account is not connected. */
 		return NULL;
 	}
 
@@ -622,6 +642,18 @@ empathy_tp_chat_get_channel (EmpathyTpChat *chat)
 	return priv->tp_chan;
 }
 
+McAccount *
+empathy_tp_chat_get_account (EmpathyTpChat *chat)
+{
+	EmpathyTpChatPriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_TP_CHAT (chat), NULL);
+
+	priv = GET_PRIV (chat);
+
+	return priv->account;
+}
+
 GList *
 empathy_tp_chat_get_pendings (EmpathyTpChat *chat)
 {
@@ -716,8 +748,8 @@ empathy_tp_chat_send (EmpathyTpChat *chat,
 }
 
 void
-empathy_tp_chat_set_state (EmpathyTpChat             *chat,
-			   TelepathyChannelChatState  state)
+empathy_tp_chat_set_state (EmpathyTpChat      *chat,
+			   TpChannelChatState  state)
 {
 	EmpathyTpChatPriv *priv;
 	GError            *error = NULL;
@@ -871,10 +903,10 @@ tp_chat_send_error_cb (DBusGProxy    *text_iface,
 }
 
 static void
-tp_chat_state_changed_cb (DBusGProxy                *chat_state_iface,
-			  guint                      handle,
-			  TelepathyChannelChatState  state,
-			  EmpathyTpChat             *chat)
+tp_chat_state_changed_cb (DBusGProxy         *chat_state_iface,
+			  guint               handle,
+			  TpChannelChatState  state,
+			  EmpathyTpChat      *chat)
 {
 	EmpathyTpChatPriv *priv;
 	EmpathyContact     *contact;
@@ -919,7 +951,7 @@ tp_chat_build_message (EmpathyTpChat *chat,
 	empathy_message_set_type (message, type);
 	empathy_message_set_sender (message, sender);
 	empathy_message_set_receiver (message, priv->user);
-	empathy_message_set_timestamp (message, (EmpathyTime) timestamp);
+	empathy_message_set_timestamp (message, timestamp);
 
 	g_object_unref (sender);
 
